@@ -63,7 +63,20 @@ START_TEST(test_get_esa_id)
 	tkm_kernel_sad_t *sad = tkm_kernel_sad_create();
 	fail_unless(sad->insert(sad, 23, 54, addr, addr, 27, 42, 50),
 				"Error inserting SAD entry");
-	fail_unless(sad->get_esa_id(sad, addr, addr, 42, 50) == 23,
+	fail_unless(sad->get_esa_id(sad, addr, addr, 42, 50, FALSE) == 23,
+				"Error getting esa id");
+	sad->destroy(sad);
+	addr->destroy(addr);
+}
+END_TEST
+
+START_TEST(test_get_esa_id_local)
+{
+	host_t *addr = host_create_from_string("127.0.0.1", 1024);
+	tkm_kernel_sad_t *sad = tkm_kernel_sad_create();
+	fail_unless(sad->insert(sad, 23, 54, addr, addr, 27, 42, 50),
+				"Error inserting SAD entry");
+	fail_unless(sad->get_esa_id(sad, addr, addr, 27, 50, TRUE) == 23,
 				"Error getting esa id");
 	sad->destroy(sad);
 	addr->destroy(addr);
@@ -74,44 +87,13 @@ START_TEST(test_get_esa_id_nonexistent)
 {
 	host_t *addr = host_create_from_string("127.0.0.1", 1024);
 	tkm_kernel_sad_t *sad = tkm_kernel_sad_create();
-	fail_unless(sad->get_esa_id(sad, addr, addr, 42, 50) == 0,
+	fail_unless(sad->get_esa_id(sad, addr, addr, 42, 50, FALSE) == 0,
 				"Got esa id for nonexistent SAD entry");
 	sad->destroy(sad);
 	addr->destroy(addr);
 }
 END_TEST
 
-START_TEST(test_get_other_esa_id)
-{
-	host_t *addr = host_create_from_string("127.0.0.1", 1024);
-	tkm_kernel_sad_t *sad = tkm_kernel_sad_create();
-	fail_unless(sad->insert(sad, 23, 54, addr, addr, 27, 42, 50),
-				"Error inserting SAD entry");
-	fail_unless(sad->insert(sad, 24, 54, addr, addr, 27, 42, 50),
-				"Error inserting SAD entry");
-	fail_unless(sad->get_other_esa_id(sad, 23) == 24,
-				"Error getting other esa id");
-	sad->destroy(sad);
-	addr->destroy(addr);
-}
-END_TEST
-
-START_TEST(test_get_other_esa_id_nonexistent)
-{
-	host_t *addr = host_create_from_string("127.0.0.1", 1024);
-	tkm_kernel_sad_t *sad = tkm_kernel_sad_create();
-	fail_unless(sad->get_other_esa_id(sad, 1) == 0,
-				"Got other esa id for nonexistent SAD entry");
-	fail_unless(sad->insert(sad, 23, 54, addr, addr, 27, 42, 50),
-				"Error inserting SAD entry");
-	fail_unless(sad->get_other_esa_id(sad, 23) == 0,
-				"Got own esa id");
-
-	sad->destroy(sad);
-	addr->destroy(addr);
-}
-END_TEST
-
 START_TEST(test_get_dst_host)
 {
 	host_t *addr = host_create_from_string("127.0.0.1", 1024);
@@ -179,14 +161,10 @@ Suite *make_kernel_sad_tests()
 
 	tc = tcase_create("get_esa_id");
 	tcase_add_test(tc, test_get_esa_id);
+	tcase_add_test(tc, test_get_esa_id_local);
 	tcase_add_test(tc, test_get_esa_id_nonexistent);
 	suite_add_tcase(s, tc);
 
-	tc = tcase_create("get_other_esa_id");
-	tcase_add_test(tc, test_get_other_esa_id);
-	tcase_add_test(tc, test_get_other_esa_id_nonexistent);
-	suite_add_tcase(s, tc);
-
 	tc = tcase_create("get_dst_host");
 	tcase_add_test(tc, test_get_dst_host);
 	tcase_add_test(tc, test_get_dst_host_nonexistent);
