@@ -238,7 +238,8 @@ static bool build_auth(private_pretend_auth_t *this,
 	}
 	keymat = (keymat_v2_t*)ike_sa->get_keymat(ike_sa);
 	if (!keymat->get_auth_octets(keymat, TRUE, this->ike_init,
-								 this->nonce, this->id, this->reserved, &octets))
+								 this->nonce, this->id, this->reserved,
+								 &octets, NULL))
 	{
 		private->destroy(private);
 		return FALSE;
