@@ -144,6 +144,10 @@ enum cred_encoding_part_t {
 	CRED_PART_PKCS10_ASN1_DER,
 	/** a PGP encoded certificate */
 	CRED_PART_PGP_CERT,
+	/** a DER encoded EdDSA public key */
+	CRED_PART_EDDSA_PUB_ASN1_DER,
+	/** a DER encoded EdDSA private key */
+	CRED_PART_EDDSA_PRIV_ASN1_DER,
 	/** a DER encoded BLISS public key */
 	CRED_PART_BLISS_PUB_ASN1_DER,
 	/** a DER encoded BLISS private key */
